require 'fog'

class FogClient
  attr_reader :type, :config, :secret_access_key, :access_key_id

  def initialize(type, config)
    @type, @config = type, config

    creds = config.credentials
    @secret_access_key, @access_key_id = creds.secret_access_key, creds.access_key_id
  end

  def credentials(op)
    if op == :elb || op == :iam
      return {
          :aws_secret_access_key=>secret_access_key,
          :aws_access_key_id=>access_key_id,
          :connection_options => {
              :ssl_verify_peer => false
          }
      }.merge(uri_params(op_endpoint(op)))
    end

    endpoint = op_endpoint(op)

    {
        :provider=>'AWS',
        :aws_secret_access_key=>secret_access_key,
        :aws_access_key_id=>access_key_id,
        :connection_options=>{:ssl_verify_peer=>false}.merge(uri_params(endpoint))
    }.merge(endpoint_params(endpoint))
  end

  def op_endpoint(op)
    config.respond_to?(op) ? config.send(op) : nil
  end

  def endpoint_params(url)
    if url
      {:endpoint=>url}
    else
      {}
    end
  end

  def uri_params(url)
    if url
      uri = URI.parse url
      {:scheme => uri.scheme,
       :host => uri.host,
       :port => uri.port,
       :path => uri.path,
      }
    else
      {}
    end
  end

  def dns
    Fog::DNS.new(credentials(:dns))
  end

  def compute
    Fog::Compute.new(credentials(:compute))
  end

  def iam
    Fog::AWS::IAM.new(credentials(:iam))
  end

  def elb
    Fog::AWS::ELB.new(credentials(:elb))
  end
end

class Fog::Compute::AWS::Servers < Fog::Collection
  def running(&block)
    if block_given?
      all.select{|s| s.ready? && (yield s)}
    else
      all.select{|s|s.ready?}
    end
  end

  # Get servers matching given regex
  def by_tag(tag, input)
    if input.is_a? String
      running {|s| s.tags[tag] == input}
    elsif input.is_a? Regexp
      running {|s| s.tags[tag] =~ input}
    else
      raise "Unknown input object type: #{input.class}"
    end
  end

  def method_missing(name, *args, &block)
    if name.match(/^by_/) && args.size == 1
      by_tag(name[3..-1], args[0])
    else
      super(name, *args, &block)
    end
  end
end

